/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <curl/curl.h>
#include <dlog.h>
#include "$(appName).h"
#include "service/http_query.h"

static CURL *__curl_h = NULL;

/*
 * @brief Initializes the CURL engine responsible for sending the requests
 * to the web service over the HTTP protocol.
 * @return: The function returns 'true' if the CURL engine was initialized successfully,
 * otherwise 'false' is returned.
 */
bool http_query_init(void)
{
	if (__curl_h) {
		dlog_print(DLOG_INFO, LOG_TAG, "HTTP engine already initialized.");
		return true;
	}

	if (curl_global_init(CURL_GLOBAL_NOTHING) != 0) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function curl_global_init() failed.");
		return false;
	}

	__curl_h = curl_easy_init();

	if (!__curl_h) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function curl_easy_init() failed.");
		http_query_deinit();
		return false;
	}

	dlog_print(DLOG_INFO, LOG_TAG, "HTTP engine initialized successfully.");

	return true;
}

/*
 * @brief Finalize the CURL engine responsible for sending the requests
 * to the web service over the HTTP protocol.
 */
void http_query_deinit(void)
{
	curl_easy_cleanup(__curl_h);
	curl_global_cleanup();
	__curl_h = NULL;
}

/*
 * @brief Sets the URL as a requests target and attaches the response receive callback.
 * @param[in] url The URL string.
 * @param[in] callback The callback function to be invoked when the web service responds to the request.
 * @param[in] user_data The user data to be passed to the 'callback' function.
 * @return: The function returns 'true' if the url and callback functions were set successfully,
 * otherwise 'false' is returned.
 */
bool http_query_set_url(const char *url, http_query_data_received_cb callback, void *user_data)
{
	if (!__curl_h) {
		dlog_print(DLOG_ERROR, LOG_TAG, "HTTP engine not initialized.");
		return false;
	}

	if (curl_easy_setopt(__curl_h, CURLOPT_URL, url) != CURLE_OK) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function curl_easy_setopt() failed to set URL.");
		return false;
	}

	if (curl_easy_setopt(__curl_h, CURLOPT_WRITEFUNCTION, callback) != CURLE_OK) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function curl_easy_setopt() failed to set callback for data receive.");
		return false;
	}

	if (curl_easy_setopt(__curl_h, CURLOPT_WRITEDATA, user_data) != CURLE_OK) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function curl_easy_setopt() failed to write callback data pointer.");
		return false;
	}

	dlog_print(DLOG_INFO, LOG_TAG, "HTTP engine URL set to %s", url);

	return true;
}

/*
 * @brief Executes the request to the web service over the HTTP protocol.
 * @return: The function returns 'true' if the request was executed successfully,
 * otherwise 'false' is returned.
 */
bool http_query_do(void)
{
	if (!__curl_h) {
		dlog_print(DLOG_ERROR, LOG_TAG, "HTTP engine not initialized.");
		return false;
	}

	CURLcode ret = curl_easy_perform(__curl_h);
	if (ret != CURLE_OK) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function curl_easy_perform() failed with error = %d.", ret);
		return false;
	}

	dlog_print(DLOG_INFO, LOG_TAG, "HTTP query submitted.");

	return true;
}
