/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <dlog.h>
#include <stdio.h>
#include <stdlib.h>
#include "$(appName).h"
#include "service/weather_query.h"
#include "service/http_query.h"

/*
 * @brief Internal callback function invoked when the weather web service responds
 * to the previously sent request.
 * @param[in] ptr The pointer to the response stream in XML format.
 * @param[in] size The size of the buffer pointed by the 'ptr'.
 * @param[in] nmemb The 'size' multiplier.
 * @param[in] userdata The user data passed to the http_query_set_url() function.
 * @return: The function returns the size of the received data.
 */
static size_t __weather_data_received_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
{
	xml_weather_t *weather = (xml_weather_t *)userdata;
	size_t rcv_size = size * nmemb;

	dlog_print(DLOG_INFO, LOG_TAG, "HTTP response received. Size = %d bytes", rcv_size);

	if (!data_get_weather((const char *)ptr, rcv_size, &weather))
		dlog_print(DLOG_ERROR, LOG_TAG, "Function data_xml_weather_parse() failed.");

	return rcv_size;
}

/*
 * @brief Initializes the weather data request channel from the web service
 * over the HTTP protocol.
 * @return: The function returns 'true' if the channel was initialized successfully,
 * otherwise 'false' is returned.
 */
bool weather_init(void)
{
	return http_query_init();
}

/*
 * @brief Finalize the weather data request channel.
 */
void weather_deinit(void)
{
	http_query_deinit();
}

/*
 * @brief Function sends the request to the web service for weather data acquisition.
 * @param[in] latitude The latitude of the geographical location to query for weather data.
 * @param[in] longitude The longitude of the geographical location to query for weather data/
 * @param[in] callback The callback function to be invoked when the web service
 * responds with the data requested.
 * @return: The function returns 'true' if the request was sent successfully,
 * otherwise 'false' is returned.
 */
bool weather_query(double latitude, double longitude, weather_data_read_cb callback)
{
	xml_weather_t *weather = (xml_weather_t *)calloc(1, sizeof(xml_weather_t));
	char buff[1024] = {0,};

	snprintf(buff,
				sizeof(buff),
				"http://api.openweathermap.org/data/2.5/weather?lat=%.6f&lon=%.6f&units=metric&mode=xml&appid=9b7b379fc3a69813cc1dd1c5891c2e45",
				latitude,
				longitude);

	if (!http_query_set_url(buff,
							__weather_data_received_cb,
							(void *)weather)) {
		data_dispose_weather(weather);
		return false;
	}

	if (!http_query_do()) {
		data_dispose_weather(weather);
		return false;
	}

	if (callback)
		callback(weather);

	return true;
}
