/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <watch_app_efl.h>
#include "view.h"
#include "view_defines.h"

#define DEFAULT_TXT_BUFF_SIZE 128
#define COLOR_HOT_R 155
#define COLOR_HOT_G 61
#define COLOR_HOT_B 58
#define COLOR_HOT_A 255
#define COLOR_COLD_R 98
#define COLOR_COLD_G 184
#define COLOR_COLD_B 194
#define COLOR_COLD_A 255

static struct view_info {
	Evas_Object *win;
	Evas_Object *layout;
	Evas_Object *image;
	Evas_Object *cloud1;
	Evas_Object *cloud2;
	Evas_Object *cloud3;
	int w;
	int h;
} s_info = {
	.win = NULL,
	.layout = NULL,
	.image = NULL,
	.cloud1 = NULL,
	.cloud2 = NULL,
	.cloud3 = NULL,
	.w = 0,
	.h = 0,
};

static void _set_text(const char *part_name, const char *text, bool upper_case);
static void _emit_signal(const char *part_name, const char *signal_name);
static char *_create_resource_path(const char *file_name);
static Evas_Object *_create_layout(void);
static Evas_Object *_create_image(const char *part_name);
static void _set_background(int temp);
static void _set_temperature(int value);
static void _set_image(Evas_Object *image, const char *file_name, int temp);
static bool _create_clouds(void);

/*
 * @brief Creates the application's UI with window's width and height preset.
 * @param[in] width The width of the window.
 * @param[in] height The height of the window.
 */
void view_create_with_size(int width, int height)
{
	s_info.w = width;
	s_info.h = height;

	view_create();
}

/*
 * @brief Create Essential Object window and layout
 */
void view_create(void)
{
	s_info.win = view_create_win(PACKAGE);
	if (!s_info.win) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return;
	}

	s_info.layout = _create_layout();
	if (!s_info.layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create main layout.");
		return;
	}

	s_info.image = _create_image(PART_IMAGE);
	if (!s_info.image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create weather image.");
		return;
	}

	if (!_create_clouds()) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create animated clouds.");
		return;
	}

	evas_object_show(s_info.win);
}

/*
 * @brief Makes a basic window named with package name.
 * @param[pkg_name]: Name of the window.
 * @return: The function returns the window's object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;
	int ret;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate window
	 */
	ret = watch_app_get_elm_win(&win);
	if (ret != APP_ERROR_NONE)
		return NULL;

	elm_win_title_set(win, pkg_name);
	elm_win_borderless_set(win, EINA_TRUE);
	elm_win_alpha_set(win, EINA_FALSE);
	elm_win_indicator_mode_set(win, ELM_WIN_INDICATOR_HIDE);
	elm_win_indicator_opacity_set(win, ELM_WIN_INDICATOR_BG_TRANSPARENT);
	elm_win_prop_focus_skip_set(win, EINA_TRUE);
	elm_win_role_set(win, "no-effect");

	evas_object_resize(win, s_info.w, s_info.h);

	return win;
}

/*
 * @brief Makes and sets a layout to the part
 * @param[parent]: Object to which you want to set this layout
 * @param[file_path]: File path of EDJ will be used
 * @param[group_name]: Group name in EDJ that you want to set to layout
 * @param[part_name]: Part name to which you want to set this layout
 * @return: The function returns the layout's object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_layout_for_part(Evas_Object *parent, char *file_path, char *group_name, char *part_name)
{
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	layout = elm_layout_add(parent);
	elm_layout_file_set(layout, file_path, group_name);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	evas_object_show(layout);

	elm_object_part_content_set(parent, part_name, layout);

	return layout;
}

/*
 * @brief Destroys the main window.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/*
 * @brief Displays all the weather related information.
 * @param[in] temperature The current temperature value.
 * @param[in] weather_fname The name of the image file representing the current weather.
 */
void view_set_weather(int temperature, const char *weather_fname)
{
	char fname[PATH_MAX] = {0,};

	snprintf(fname, sizeof(fname), "%s.png", weather_fname);

	_set_background(temperature);
	_set_image(s_info.image, fname, temperature);
	_set_temperature(temperature);
	_set_image(s_info.cloud1, "cloud1.png", temperature);
	_set_image(s_info.cloud2, "cloud1.png", temperature);
	_set_image(s_info.cloud3, "cloud2.png", temperature);

	if (temperature > 0)
		_emit_signal(PART_DAY_OF_WEEK, SIGNAL_DAY_OF_WEEK_HOT_SET);
	else
		_emit_signal(PART_DAY_OF_WEEK, SIGNAL_DAY_OF_WEEK_COLD_SET);
}

/*
 * @brief Displays the city name in capitals.
 * @param[in] city The city name to display.
 */
void view_set_city(const char *city, const char *country)
{
	_set_text(PART_CITY, city, true);

	if(!country)
	{
		dlog_print(DLOG_ERROR, LOG_TAG, "Country is null. Location is not set!");
		return;
	}

	if (strncasecmp(country, "none", strlen("none")) == 0)
		_set_text(PART_COUNTRY, " ", true);
	else
		_set_text(PART_COUNTRY, country, true);
}

/*
 * @brief Displays the time represented by single circular indicator.
 * @param[in] hour The hour component of the time to display.
 * @param[in] minute The minute component of the time to display.
 */
void view_set_time(int hour, int minute)
{
	Edje_Message_Int_Set *msg = NULL;

	if (!s_info.layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Cannot set time to nonexistent layout.");
		return;
	}

	msg = (Edje_Message_Int_Set *)malloc(sizeof(Edje_Message_Int_Set) + 2 * sizeof(int));
	if (!msg) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to allocate memory for time message.");
		return;
	}

	if (hour >= 12)
		hour -= 12;

	msg->count = 2;
	msg->val[0] = hour;
	msg->val[1] = minute;

	edje_object_message_send(elm_layout_edje_get(s_info.layout), EDJE_MESSAGE_INT_SET, MSG_ID_TIME, msg);
	free(msg);
}

/*
 * @brief Displays the current date.
 * @param[in] day The day component of the date to display.
 * @param[in] month The month component of the date to display.
 * @param[in] year The year component of the date to display.
 * @param[in] dow The number of the day of the week.
 */
void view_set_date(int day, int month, int year, int dow)
{
	const char *dow_str[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	char date_str[11] = {0,};

	snprintf(date_str, sizeof(date_str), "%02d %02d %d", day, month, year);

	_set_text(PART_DAY_OF_WEEK, dow_str[dow - 1], true);
	_set_text(PART_DATE, date_str, false);
}

/*
 * @brief Toggles the clouds animation on/off.
 * @param[in] visible The clouds visibility sate to be set.
 */
void view_toggle_clouds(bool visible)
{
	if (visible) {
		_emit_signal(PART_CLOUD_1, SIGNAL_CLOUDS_ANIMATION_RESET);
		_emit_signal(PART_CLOUD_2, SIGNAL_CLOUDS_ANIMATION_RESET);
		_emit_signal(PART_CLOUD_3, SIGNAL_CLOUDS_ANIMATION_RESET);
	} else {
		_emit_signal(PART_CLOUD_1, SIGNAL_CLOUDS_ANIMATION_STOP);
		_emit_signal(PART_CLOUD_2, SIGNAL_CLOUDS_ANIMATION_STOP);
		_emit_signal(PART_CLOUD_3, SIGNAL_CLOUDS_ANIMATION_STOP);
	}
}

/*
 * @brief Internal function sets the text to the layout's text part.
 * @param[in] part_name The name of the text part to be affected.
 * @param[in] text The text to be displayed in the text part.
 * @param[in] upper_case The flag determines whether the text shall be displayed in capitals.
 */
static void _set_text(const char *part_name, const char *text, bool upper_case)
{
	char *temp = NULL;

	if (!s_info.layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Cannot set text to nonexistent layout.");
		return;
	}

	if (text) {
		temp = strdup(text);

		if (upper_case)
			eina_str_toupper(&temp);
	}

	elm_object_part_text_set(s_info.layout, part_name, temp);
	free(temp);
}

/*
 * @brief Internal function emits the signal to the layout's part.
 * @param[in] part_name The name of the part to be affected.
 * @param[in] signal_name The name of the signal to be sent to the layout's part.
 */
static void _emit_signal(const char *part_name, const char *signal_name)
{
	if (!s_info.layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Cannot emit signal to nonexistent layout.");
		return;
	}

	elm_object_signal_emit(s_info.layout, signal_name, part_name);
}

/*
 * @brief Internal function which creates path to the given resource file
 * by concatenation of the basic resource path and the given file_name.
 * @param[file_name]: File name or path relative to the resource directory.
 * @return: The absolute path to the resource with given file_name is returned.
 */
static char *_create_resource_path(const char *file_name)
{
	static char res_path_buff[PATH_MAX] = {0,};
	char *res_path = NULL;

	res_path = app_get_resource_path();
	if (res_path == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get resource path.");
		return NULL;
	}

	snprintf(res_path_buff, PATH_MAX, "%s%s", res_path, file_name);
	free(res_path);

	return &res_path_buff[0];
}

/*
 * @brief Internal function which creates the application's layout.
 * @return: The function returns the layout's object if it was created successfully,
 * otherwise NULL is returned.
 */
static Evas_Object *_create_layout(void)
{
	char *edj_path = NULL;
	Evas_Object *layout = NULL;

	edj_path = _create_resource_path(MAIN_EDJ);

	layout = view_create_layout_for_part(s_info.win, edj_path, MAIN_GRP, "default");
	if (!layout)
		return NULL;

	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_min_set(layout, s_info.w, s_info.h);
	evas_object_resize(layout, s_info.w, s_info.h);

	evas_object_show(layout);

	return layout;
}

/*
 * @brief Internal function which creates the image widget.
 * @param[in] part_name The target part's name where the image is to be set.
 * @return: The function returns the image's object if it was created successfully,
 * otherwise NULL is returned.
 */
static Evas_Object *_create_image(const char *part_name)
{
	Evas_Object *image = elm_image_add(s_info.layout);
	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "function elm_image_add() failed.");
		return NULL;
	}

	elm_object_part_content_set(s_info.layout, part_name, image);
	evas_object_show(image);

	return image;
}

/*
 * @brief Internal function which sets the background image based on the current temperature.
 * @param[in] temp The current temperature which is used as a basis for the background image setting.
 * If the @temp is greater than 0, then hot background is applied. Otherwise, the cold one
 * is applied.
 */
static void _set_background(int temp)
{
	if (temp > 0) {
		_emit_signal(PART_BACKGROUND, SIGNAL_BACKGROUND_HOT_SET);
		_emit_signal(PART_FRAME, SIGNAL_FRAME_HOT_SET);
	} else {
		_emit_signal(PART_BACKGROUND, SIGNAL_BACKGROUND_COLD_SET);
		_emit_signal(PART_FRAME, SIGNAL_FRAME_COLD_SET);
	}
}

/*
 * @brief Internal function which displays the temperature value.
 * @param[in] value The value of the temperature to display.
 */
static void _set_temperature(int value)
{
	char text[DEFAULT_TXT_BUFF_SIZE] = {0,};

	snprintf(text, DEFAULT_TXT_BUFF_SIZE, "%d°", abs(value));

	_set_text(PART_TEMPERATURE, text, false);

	if (value >= 0) {
		_emit_signal(PART_TEMPERATURE_SIGN, SIGNAL_TEMPERATURE_SIGN_HIDE);
		_set_text(PART_TEMPERATURE_TEXT, "HOT", true);
	} else {
		_emit_signal(PART_TEMPERATURE_SIGN, SIGNAL_TEMPERATURE_SIGN_SHOW);
		_set_text(PART_TEMPERATURE_TEXT, "COLD", true);
	}
}

/*
 * @brief Internal function which sets the image to be displayed. The image color is set based on the current temperature.
 * @param[in] image The image object to be altered.
 * @param[in] file_name The name of the file to be displayed. The file must be stored in res/images directory.
 * @param[in] temp The current temperature which is used as a basis for the image's color setting.
 * If the @temp is greater than 0, then hot color is applied. Otherwise, the cold one
 * is applied.
 */
static void _set_image(Evas_Object *image, const char *file_name, int temp)
{
	char *img_path = NULL;
	char buff[PATH_MAX] = {0,};

	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Cannot set image to nonexistent image object.");
		return;
	}

	snprintf(buff, sizeof(buff), "images/%s", file_name);
	img_path = _create_resource_path(buff);

	if (temp > 0)
		evas_object_color_set(image, COLOR_HOT_R, COLOR_HOT_G, COLOR_HOT_B, COLOR_HOT_A);
	else
		evas_object_color_set(image, COLOR_COLD_R, COLOR_COLD_G, COLOR_COLD_B, COLOR_COLD_A);

	if (!elm_image_file_set(image, img_path, NULL))
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_image_file_set() failed.");
}

/*
 * @brief Internal function which creates three animated cloud images.
 * @return: The function returns 'true' if the clouds were created successfully,
 * otherwise 'false' is returned.
 */
static bool _create_clouds(void)
{
	s_info.cloud1 = _create_image(PART_CLOUD_1);
	if (!s_info.cloud1) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create cloud-1 image.");
		return false;
	}

	s_info.cloud2 = _create_image(PART_CLOUD_2);
	if (!s_info.cloud2) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create cloud-2 image.");
		return false;
	}

	s_info.cloud3 = _create_image(PART_CLOUD_3);
	if (!s_info.cloud3) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create cloud-3 image.");
		return false;
	}

	view_toggle_clouds(true);

	return true;
}
